# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.
from PIL import Image
import threading
from weakref import WeakValueDictionary
from pathlib import Path
import os

def determine_font_path_automatically(font_file = 'Times New Roman.ttf'):
    font_dirs = [
        '/usr/share/fonts',          
        '/usr/local/share/fonts',    
        '/Library/Fonts',           
    ]
        
    for font_dir in font_dirs:
        font_path = os.path.join(font_dir, font_file)
        if os.path.isfile(font_path):
            return font_path

    return None

assets = Path(__file__).parent.parent / 'assets'
font = str(assets / 'font' / 'NotoSansKR-Bold.ttf')



async def change_font(x,genshin_font = False, comf = False, font_path = None):
    global font
    if font_path is None:
        if genshin_font:
            font = str(assets / 'font' / 'Genshin_Impact.ttf')
        elif comf:
            font = str(assets / 'font' / 'font_hsr.ttf')
        else:
            font = str(assets / 'font' / 'NotoSansKR-Bold.ttf')
    else:
        font_path = os.path.abspath(font_path)
        if os.path.isfile(font_path):
            font = font_path 
        else:
            font_path = determine_font_path_automatically(font_path)
            if font_path is None:
                font = str(assets / 'font' / 'NotoSansKR-Bold.ttf')

mapping = {
    'total_bg': assets/'bg'/'bg.png',
    'bg_charters': assets/'bg'/'bg_charters.png',
    'frame_charters': assets/'bg'/'frame_charters.png',
    'maska_charters': assets/'bg'/'maska.png',

    'strs_1': assets/'stars'/'strs_1.png',
    'strs_2': assets/'stars'/'strs_2.png',
    'strs_3': assets/'stars'/'strs_3.png',
    'strs_4': assets/'stars'/'strs_4.png',
    'strs_5': assets/'stars'/'strs_5.png',

    'Closed': assets/'const'/'Closed.png',
    'Lock': assets/'const'/'Lock.png',
    'open': assets/'const'/'open.png',

    'bg_lc_total': assets/'lc'/'bg.png',
    'bg_lc': assets/'lc'/'bg_lc.png',
    'maska_lc': assets/'lc'/'maska.png',

    'logo': assets/'LOGO.png',
    'uid': assets/'UID.png',
    'STATS': assets/'STATS.png',

    'ARTIFACT': assets/'ARTIFACT.png',

    'bg_main': assets/'Tallants'/'bg_main.png',
    'bg_dop': assets/'Tallants'/'bg_dop.png',
    'bg_talants': assets/'Tallants'/'bg.png',
    'count_talants': assets/'Tallants'/'count.png',
    'dop_0': assets/'Tallants'/'dop_0.png',
    'dop_1': assets/'Tallants'/'dop_1.png',
    'dop_2': assets/'Tallants'/'dop_2.png',
    'dop_3': assets/'Tallants'/'dop_3.png', 

    'sets_bg': assets/'Sets'/'bg.png',
    'sets_count': assets/'Sets'/'count.png',

    #TEAMPLE TWO

    "Knight": assets/'teample_two_new'/'path'/'Knight.png',
    "Mage": assets/'teample_two_new'/'path'/'Mage.png',
    "Priest": assets/'teample_two_new'/'path'/'Priest.png',
    "Rogue": assets/'teample_two_new'/'path'/'Rogue.png',
    "Shaman": assets/'teample_two_new'/'path'/'Shaman.png',
    "Warlock": assets/'teample_two_new'/'path'/'Warlock.png',
    "Warrior": assets/'teample_two_new'/'path'/'Warrior.png',

    #TEAMPLE TREE
    "shadow_bg": assets/'teample_tree'/'bg'/'shadow_bg.png',
    "splash_art": assets/'teample_tree'/'bg'/'splash_art.png',
    "shadow_art": assets/'teample_tree'/'bg'/'shadow_art.png',
    "art_frame": assets/'teample_tree'/'bg'/'art_frame.png',
    "overlay": assets/'teample_tree'/'bg'/'overlay.png',

    "lc_3_stars": assets/'teample_tree'/'lc'/'3_stars.png',
    "lc_4_stars": assets/'teample_tree'/'lc'/'4_stars.png',
    "lc_5_stars": assets/'teample_tree'/'lc'/'5_stars.png',
    "blink": assets/'teample_tree'/'lc'/'blink.png',
    "lc_info": assets/'teample_tree'/'lc'/'info.png',

    "closed_main": assets/'teample_tree'/'skills'/'closed_main.png',
    "main_skills": assets/'teample_tree'/'skills'/'main.png',
    "closed_trees": assets/'teample_tree'/'skills'/'closed_trees.png',
    "count_tree": assets/'teample_tree'/'skills'/'count_tree.png',
    "dop": assets/'teample_tree'/'skills'/'dop.png',
    "open_trees": assets/'teample_tree'/'skills'/'open_trees.png',

    "ON_const": assets/'teample_tree'/'const'/'ON.png',
    "OFF_const": assets/'teample_tree'/'const'/'OFF.png',
    "CLOSED_const": assets/'teample_tree'/'const'/'CLOSED.png',

    "artifact_tree": assets/'teample_tree'/'artifact'/'artifact.png',
    "sets_tree": assets/'teample_tree'/'artifact'/'sets.png',


    "frame_art_1": assets/'teample_tree'/'artifact'/'frame_art_1.png',
    "frame_art_2": assets/'teample_tree'/'artifact'/'frame_art_2.png',
    "frame_art_3": assets/'teample_tree'/'artifact'/'frame_art_3.png',
    "frame_art_4": assets/'teample_tree'/'artifact'/'frame_art_4.png',
    "frame_art_5": assets/'teample_tree'/'artifact'/'frame_art_5.png',

    "name_banners": assets/'teample_tree'/'name_banners.png',
    "stars_five": assets/'teample_tree'/'stars_five.png',
    "stars_four": assets/'teample_tree'/'stars_four.png',

    "icon": assets/"icon.png",
    "icon_maska": assets/"icon_maska.png",


    "seeleland": assets/"seeleland.png",
    "seeleland_v2": assets/"seeleland_v2.png",

    #TEAMPLE PROFILE
    "default_bg": assets/'teample_profile'/"bg"/'default_bg.png',
    "bg_shadow": assets/'teample_profile'/"bg"/'shadow.png',
    "name_frame":assets/'teample_profile'/"bg"/'name_frame.png',
    "total_bg_frame":assets/'teample_profile'/"bg"/'frame.png',

    "charter_icon_bg":assets/'teample_profile'/"charters"/'bg.png',
    "frame_stars_charters_4":assets/'teample_profile'/"charters"/'frame_stars_4.png',
    "frame_stars_charters_5":assets/'teample_profile'/"charters"/'frame_stars_5.png',
    "stats_frame_icon":assets/'teample_profile'/"charters"/'stats_frame_icon.png',
    "ups":assets/'teample_profile'/"charters"/'ups.png',

    "element_electro":assets/'teample_profile'/"element"/'electro.png',
    "element_fire":assets/'teample_profile'/"element"/'fire.png',
    "element_ice":assets/'teample_profile'/"element"/'ice.png',
    "element_IMAGINARY":assets/'teample_profile'/"element"/'IMAGINARY.png',
    "element_psyhical":assets/'teample_profile'/"element"/'psyhical.png',
    "element_quantom":assets/'teample_profile'/"element"/'quantom.png',
    "element_wind":assets/'teample_profile'/"element"/'wind.png',


    #TEAMPLE FOUR

    'ANEMO': assets/'teample_enkcard'/'background'/'ANEMO2.png',
    'CRYO': assets/'teample_enkcard'/'background'/'CRYO.png',
    'Electro': assets/'teample_enkcard'/'background'/'Electro.png',
    'imaginary': assets/'teample_enkcard'/'background'/'imaginary.png',
    'psyhical': assets/'teample_enkcard'/'background'/'psyhical.png',
    'PYRO': assets/'teample_enkcard'/'background'/'PYRO2.png',
    'quantom': assets/'teample_enkcard'/'background'/'quantom.png',

    'CONST_CLOSED': assets/'teample_enkcard'/'const'/'Closed.png',
    'CONST_CLOSED_LOCK': assets/'teample_enkcard'/'const'/'Lock.png',

    'CONST_ANEMO': assets/'teample_enkcard'/'const'/'const_wind.png',
    'CONST_CRYO': assets/'teample_enkcard'/'const'/'const_ice.png',
    'CONST_Electro': assets/'teample_enkcard'/'const'/'const_electro.png',
    'CONST_imaginary': assets/'teample_enkcard'/'const'/'const_imaginary.png',
    'CONST_psyhical': assets/'teample_enkcard'/'const'/'const_psyhical.png',
    'CONST_PYRO': assets/'teample_enkcard'/'const'/'const_pyro.png',
    'CONST_quantom': assets/'teample_enkcard'/'const'/'const_quantom.png',

    'maska_background': assets/'teample_enkcard'/'background'/'maska.png',
    
    'shadow_enc': assets/'teample_enkcard'/'background'/'shadow.png',
    'effect_soft': assets/'teample_enkcard'/'background'/'soft.png',
    'effect_overlay': assets/'teample_enkcard'/'background'/'overlay.png',

    'lc_frame': assets/'teample_enkcard'/'lc_frame.png',

    'strs_5_shadow': assets/'teample_enkcard'/'stars'/'strs_5.png',
    'strs_4_shadow': assets/'teample_enkcard'/'stars'/'strs_4.png',
    'strs_3_shadow': assets/'teample_enkcard'/'stars'/'strs_3.png',

    'artifact_bg': assets/'teample_enkcard'/'artifact'/'bg.png',
    'artifact_mask': assets/'teample_enkcard'/'artifact'/'mask.png',
    'artifact_count': assets/'teample_enkcard'/'artifact'/'count.png',
    'artifact_sets_count': assets/'teample_enkcard'/'artifact'/'sets_count.png',
    'artifact_sets': assets/'teample_enkcard'/'artifact'/'set.png',



    'talants_bg': assets/'teample_enkcard'/'talants'/'bg.png',
    'talants_count': assets/'teample_enkcard'/'talants'/'count.png',
    'dop_stats': assets/'teample_enkcard'/'talants'/'dop_stats.png',


    'cryo_stats': assets/'teample_enkcard'/'talants'/'cryo_stats.png',
    'electro_stats': assets/'teample_enkcard'/'talants'/'electro_stats.png',
    'imaginari_stats': assets/'teample_enkcard'/'talants'/'imaginari_stats.png',
    'psyhical_stats': assets/'teample_enkcard'/'talants'/'psyhical_stats.png',
    'pyro_stats': assets/'teample_enkcard'/'talants'/'pyro_stats.png',
    'quantom_stats': assets/'teample_enkcard'/'talants'/'quantom_stats.png',
    'wind_stats': assets/'teample_enkcard'/'talants'/'wind_stats.png',


    'stats_frame_enc': assets/'teample_enkcard'/'stats'/'frame.png',


    'relict_total_bg': assets/'relict'/'total_bg.png',
    'relict_stats': assets/'relict'/'stats.png',

    'relict_1_stars': assets/'relict'/'1_stars.png',
    'relict_2_stars': assets/'relict'/'2_stars.png',
    'relict_3_stars': assets/'relict'/'3_stars.png',
    'relict_4_stars': assets/'relict'/'4_stars.png',
    'relict_5_stars': assets/'relict'/'5_stars.png',

    
    #TEAMPLE FIVE
    'mask_bg_five': assets/'teample_teams'/'background'/'mask.png',
    'shadow_bg_five': assets/'teample_teams'/'background'/'shadow.png',
    'blink_bg_five': assets/'teample_teams'/'background'/'blink.png',
    'stars_5_frame_up': assets/'teample_teams'/'background'/'5_stars_frame_up.png',
    'stars_5_frame': assets/'teample_teams'/'background'/'5_stars_frame.png',
    'stars_4_frame_up': assets/'teample_teams'/'background'/'4_stars_frame_up.png',
    'stars_4_frame': assets/'teample_teams'/'background'/'4_stars_frame.png',
    
    'total_cards_bg':assets/'teample_teams'/'background'/'total_cards_bg.png',
    
    
    'shadow_3_lc': assets/'teample_teams'/'lk'/'3_shadow_lc.png',
    'star_3_frame': assets/'teample_teams'/'lk'/'3_star_frame.png',
    'star_3_frame_lc': assets/'teample_teams'/'lk'/'3_star_frame_lc.png',
    
    'shadow_4_lc': assets/'teample_teams'/'lk'/'4_shadow_lc.png',
    'star_4_frame': assets/'teample_teams'/'lk'/'4_star_frame.png',
    'star_4_frame_lc': assets/'teample_teams'/'lk'/'4_star_frame_lc.png',
    
    'shadow_5_lc': assets/'teample_teams'/'lk'/'5_shadow_lc.png',
    'star_5_frame': assets/'teample_teams'/'lk'/'5_star_frame.png',
    'star_5_frame_lc': assets/'teample_teams'/'lk'/'5_star_frame_lc.png',
    
    'blic_lc': assets/'teample_teams'/'lk'/'blic.png',
    'frame_lc': assets/'teample_teams'/'lk'/'frame_lc.png',
    'mask_lc_all': assets/'teample_teams'/'lk'/'mask.png',
    'maska_lc_teams': assets/'teample_teams'/'lk'/'maska_lc.png',
    'ranket': assets/'teample_teams'/'lk'/'ranket.png',
    'stats_lc': assets/'teample_teams'/'lk'/'stats.png',
    'def_bg': assets/'teample_teams'/'lk'/'def_bg.png',
    'dark': assets/'teample_teams'/'lk'/'dark.png',
    "stars_lc": assets/'teample_teams'/'lk'/'stars.png',
    
    'const_five': assets/'teample_teams'/'const'/'bg.png',
    'bg_dop_skills': assets/'teample_teams'/'skills'/'bg_dop.png',
    'bg_dop_closed': assets/'teample_teams'/'skills'/'closed.png',
    
    'stars_1_frame_relict': assets/'teample_teams'/'relic'/'1_stars.png',
    'stars_2_frame_relict': assets/'teample_teams'/'relic'/'2_stars.png',
    'stars_3_frame_relict': assets/'teample_teams'/'relic'/'3_stars.png',
    'stars_4_frame_relict': assets/'teample_teams'/'relic'/'4_stars.png',
    'stars_5_frame_relict': assets/'teample_teams'/'relic'/'5_stars.png',
    'shadow_frame_relict': assets/'teample_teams'/'relic'/'shadow.png',
    
    
    'wind_five': assets/'teample_teams'/'background'/'wind.png',
    'quantom_five': assets/'teample_teams'/'background'/'quantom.png',
    'imaginary_five': assets/'teample_teams'/'background'/'imaginary.png',
    'ice_five': assets/'teample_teams'/'background'/'ice.png',
    'fire_five': assets/'teample_teams'/'background'/'fire.png',
    'electro_five': assets/'teample_teams'/'background'/'electro.png',
    'psyhical_five': assets/'teample_teams'/'background'/'psyhical.png',
    
    
    'background_default': assets/'teample_two_new'/'background'/'background_default.png',
    'background_line': assets/'teample_two_new'/'background'/'line.png',
    'background_maska_art': assets/'teample_two_new'/'background'/'maska_art.png',
    'background_maska_blur': assets/'teample_two_new'/'background'/'maska_blur.png',
    'background_overlay': assets/'teample_two_new'/'background'/'overlay.png',
    'background_shadow': assets/'teample_two_new'/'background'/'shadow.png',
    
    
    'light_cone_frame': assets/'teample_two_new'/'light_cone'/'frame.png',
    'light_cone_frame_five': assets/'teample_two_new'/'light_cone'/'frame_five.png',
    'light_cone_frame_four': assets/'teample_two_new'/'light_cone'/'frame_four.png',
    'light_cone_frame_three': assets/'teample_two_new'/'light_cone'/'frame_three.png',
    'light_cone_stats': assets/'teample_two_new'/'light_cone'/'stats.png',
    'light_cone_ups': assets/'teample_two_new'/'light_cone'/'ups.png',
    
    
    'relict_backgroundl_score_line': assets/'teample_two_new'/'relict'/'backgroundl_score_line.png',
    'relict_full_score_line': assets/'teample_two_new'/'relict'/'full_score_line.png',
    'relict_count_sets': assets/'teample_two_new'/'relict'/'count_sets.png',
    'relict_frame_line': assets/'teample_two_new'/'relict'/'frame_line.png',    
    'relict_line': assets/'teample_two_new'/'relict'/'line.png',
    'relict_frame': assets/'teample_two_new'/'relict'/'relict_frame.png',
    'relict_maska': assets/'teample_two_new'/'relict'/'relict_maska.png',
    'relict_score_frame': assets/'teample_two_new'/'relict'/'score_frame.png',
    'none_relict': assets/'teample_two_new'/'relict'/'none_relict.png',
    
    'g_five': assets/'teample_two_new'/'stars'/'g_five.png',
    'g_four': assets/'teample_two_new'/'stars'/'g_four.png',
    'g_three': assets/'teample_two_new'/'stars'/'g_three.png',
    'g_two': assets/'teample_two_new'/'stars'/'g_two.png',
    'g_one': assets/'teample_two_new'/'stars'/'g_one.png',
    
    'LOGO_GIT': assets/'teample_two_new'/'LOGO.png',
    
    'path_count': assets/'teample_two_new'/'path'/'count.png',
    'path_closed_main': assets/'teample_two_new'/'path'/'closed_main.png',
    'path_closed_dop': assets/'teample_two_new'/'path'/'closed_dop.png',
    
}


class ImageCache:
    def __init__(self):
        self.mapping = mapping
        self.cache = WeakValueDictionary()
        self.lock = threading.Lock()
        self.assets = assets

    def __dir__(self):
        return sorted(set([*globals(), *self.mapping]))

    def __getattr__(self, name):
        path = self.mapping.get(name)
        if not path:
            raise AttributeError(name)
    
        with self.lock:
            try:
                image = self.cache[name]
            except KeyError:
                self.cache[name] = image = Image.open(self.assets / path)
        
            return image


# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.

from pydantic import BaseModel
from typing import List ,Optional,Union

from PIL import Image

class Avatar(BaseModel):
    id: Optional[str]
    name: Optional[str]
    icon: Optional[str]

class SpaceInfo(BaseModel):
    pass_area_progress: Optional[int]
    light_cone_count: Optional[int]
    avatar_count: Optional[int]
    achievement_count: Optional[int]

class PlayerV2(BaseModel):
    uid: Optional[str]
    nickname: Optional[str]
    level: Optional[int]
    avatar: Avatar
    signature: Optional[str]
    friend_count: Optional[int]
    world_level: Optional[int]
    space_info: SpaceInfo



class Card(BaseModel):
    id: Optional[str]
    name: Optional[str]
    rarity: Optional[int]
    card: Image.Image
    class Config:
        arbitrary_types_allowed = True
    size: Optional[tuple]
class Settings(BaseModel):
    uid: Optional[int]
    lang: Optional[str]
    hide: Optional[bool]
    save: Optional[bool]
    background: Optional[bool]

class HSRCard(BaseModel):
    settings: Settings
    player: PlayerV2
    card: Optional[Union[List[Card], Image.Image]] 
    cards: Optional[Image.Image]
    name: Optional[str]
    id: Optional[str]
    class Config:
        arbitrary_types_allowed = True



class MainAffix(BaseModel):
    type: Optional[str]
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: Optional[float]
    display: Optional[str]
    percent: Optional[bool]

class SubAffix(BaseModel):
    type: Optional[str]
    field: Optional[str]
    name: Optional[str]
    icon: Optional[str]
    value: Optional[float]
    display: Optional[Union[str, int]]
    percent: Optional[bool]

class Relict(BaseModel):
    id: Optional[str]
    name: Optional[str]
    set_id: Optional[str]
    set_name: Optional[str]
    rarity: Optional[int]
    level: Optional[int]
    icon: Optional[str]
    main_affix: Optional[MainAffix]
    sub_affix: Optional[List[SubAffix]]

class Score(BaseModel):
    score: Optional[float]
    rank: Optional[str]
    eff: Optional[int]
    cv: Optional[float]

class RelictData(BaseModel):
    card: Image.Image
    score: Optional[Score]
    relict: Optional[Relict]
    position: Optional[int]
    class Config:
        arbitrary_types_allowed = True


class StarRailRelict(BaseModel):
    uid: Optional[int]
    card: Image.Image 
    charter_id: Optional[int]
    relict: Optional[List[RelictData]]

    class Config:
        arbitrary_types_allowed = True



# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.

from .src.tools import translation, pill, modal, openFile
from .src.generators import one, two, tree,four,five, author, profile, relicts,two_new
from honkairail import starrailapi
import asyncio,re,os,datetime

def process_input(characterImgs, characterName,characterBackground):
    if characterImgs:
        if isinstance(characterImgs, dict):
            characterImgs = {key.lower(): value for key, value in characterImgs.items()}
        else:
            raise TypeError("The charterImg parameter must be a dictionary, where the key is the name of the character, and the parameter is an image.\nExample: charterImg = {'Himeko': 'img.png'} or {'Himeko': 'img.png', 'Seele': 'img2.jpg', ...}")

    if characterBackground:
        if isinstance(characterBackground, dict):
            characterBackground = {key.lower(): value for key, value in characterBackground.items()}
        else:
            raise TypeError("The characterBackground parameter must be a dictionary, where the key is the name of the character, and the parameter is an image.\nExample: charterImg = {'Himeko': 'img.png'} or {'Himeko': 'img.png', 'Seele': 'img2.jpg', ...}")

    
    if characterName:
        if isinstance(characterName, str):
            characterName = [name.strip().lower() for name in characterName.split(",")]
        else:
            raise TypeError("The name parameter must be a string, to pass multiple names, list them separated by commas.\nExample: name = 'Himeko' or name = 'Himeko, Seele',..")
    
    return characterImgs, characterName,characterBackground


def remove_html_tags(text):
    clean_text = re.sub('<.*?>', '', text)
    return clean_text


async def saveBanner(uid, res, name):
    data = datetime.datetime.now().strftime("%d_%m_%Y %H_%M")
    path = os.path.join(os.getcwd(), "RailCard", str(uid))
    os.makedirs(path, exist_ok=True)
    file_path = os.path.join(path, f"{name}_{data}.png")
    res.save(file_path)

class MiHoMoCard():
    def __init__(self,lang = "en", characterImgs = None, characterName = None, characterBackground = None, backgroundBlur = False, hide = False, save = False, background = True, template = 3, seeleland = False, font = None):

        """
        :param lang: str, What language to receive information supported:  en, ru, vi, th, pt, kr, jp, zh, id, fr, es, de, chs, cht.
        :param characterImgs: dict, Dictionary: {"Name_charter_1": "image link","Name_charter_2": "image link",...}.
        :param characterName: str, If we want to get certain characters: "Name_charter_1,Name_charter_1,Name_charter_1" Character names must be in the same language as in the lang parameter.
        :param hide: bool, Display UID.
        :param save: bool, Save images or not.
        :param background: bool, Generate image with or without background.
        :param seeleland: bool, Get information from the site: seeleland.com (Only for 3 patterns).
        :param font: str, Name or path to the font file to replace.

        """        
        self.font = font
        self.template = template
        self.backgroundBlur = backgroundBlur
        
        
        if not int(template) in [1,2,3,4,5]:
            self.template = 3

        if not lang in translation.supportLang:
            self.lang = "en"
        else:
            self.lang = lang
        
        self.translateLang = translation.Translator(lang)
        self.background = background
        
        try:
            self.characterImgs, self.characterName,self.characterBackground = process_input(characterImgs, characterName,characterBackground)
        except Exception as e:
            print(e.message)
            return

        self.API = starrailapi.StarRailApi(lang, v = 2)
        self.save = save
        self.hide = hide
        self.img = None
        self.characterBackgroundimg = None
        self.b = None
        self.seeleland = seeleland

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        pass
    
    async def get_characterBackgroundImg(self,name,ids):
        if name in self.characterBackground:
            self.characterBackgroundimg = await pill.get_user_image(self.characterBackground[name])
        else:
            self.characterBackgroundimg = None
        
        if ids in self.characterBackground:
            self.characterBackgroundimg = await pill.get_user_image(self.characterBackground[ids])
        
    async def characterImg(self,name,ids):
        if name in self.characterImgs:
            self.img = await pill.get_user_image(self.characterImgs[name])
        else:
            self.img = None
        
        if ids in self.characterImgs:
            self.img = await pill.get_user_image(self.characterImgs[ids])
    
    async def creat(self, uid):
        if self.font is None:
            if self.template == 4:
                await openFile.change_font(self.lang, genshin_font = True)
            elif self.template == 2:
                await openFile.change_font(self.lang, comf = True)
            else:
                await openFile.change_font(self.lang)
        else:
            await openFile.change_font(self.lang, font_path = self.font)
        task = []
        data = await self.API.get_full_data(uid)
        user = {
            "settings": {
                "uid": int(uid),
                "lang": self.lang,
                "hide": self.hide,
                "save": self.save,
                "background": self.background
            },
            "player": data.player,
            "card": [],
            "cards": None,
            "name": "",
            "id": "",
        }

        
        for key in data.characters:
            
            user["name"] += f"{key.name}, "
            user["id"] += f"{key.id}, "
            
            if self.characterName:
                if not key.name.lower() in self.characterName and not str(key.id) in self.characterName:
                    continue       

            if self.characterImgs:
                await self.characterImg(key.name.lower(), str(key.id))
            if self.characterBackground:
                await self.get_characterBackgroundImg(key.name.lower(), str(key.id))
            if self.template == 1:
                task.append(one.Creat(key, self.translateLang,self.img,self.hide,int(uid),remove_html_tags(data.player.nickname),self.background).start())
            elif self.template == 3:
                task.append(tree.Creat(key, self.translateLang,self.img,self.hide,int(uid),self.seeleland,self.characterBackgroundimg,self.backgroundBlur).start())
            elif self.template == 4:
                task.append(four.Creat(key, self.translateLang,self.img,self.hide,int(uid),self.seeleland).start())
            elif self.template == 5:
                task.append(five.Creat(key, self.translateLang, self.img).start())
            else:
                task.append(two_new.Creat(key, self.translateLang,self.img,self.hide,int(uid), self.seeleland).start())

        user["card"] = await asyncio.gather(*task)

        if self.template == 5:
            user["cards"] = await five.get_bg(self.background,user["card"])

        if self.save:
            if self.template == 5:
                await saveBanner(uid,user["cards"], "total")
            for keys in user["card"]:
                await saveBanner(uid,keys["card"], keys["name"])

        return modal.HSRCard(**user)
    
    async def get_profile(self, uid, banner = None, card = False):
        data = await self.API.get_full_data(uid)
        if not self.font is None:
            await openFile.change_font(self.lang, font_path = self.font)
            
        user = {
            "settings": {
                "uid": int(uid),
                "lang": self.lang,
                "hide": self.hide,
                "save": self.save,
                "background": self.background
            },
            "player": data.player,
            "card": None,
            "name": "",
            "id": "",
        }
        for key in data.characters:
            user["name"] += f"{key.name}, "
            user["id"] += f"{key.id}, "

        if card:
            if not banner is None:
                banner = await pill.get_user_image(banner)
            user["card"] = await profile.Creat(data, self.translateLang, banner, self.hide).start()
            

        return modal.HSRCard(**user)

    async def get_relict(self,uid,charter_id, position = None):
        """
        :param position: int, Relic position from 1 to 6.
        :param charter_id: int, The character ID of the relic to get.
        """
        if not position is None:
            if position < 1 and position > 6:
                raise TypeError("Argument: position expects a number between 1 and 6")

        await openFile.change_font(self.lang, genshin_font = True)

        result = {
            "uid": uid,
            "card": None,
            "charter_id": charter_id,
            "relict": [],
        }

        data = await self.API.get_full_data(uid)
        for character in data.characters:
            if int(character.id) == int(charter_id):
                if not position is None:
                    for z in character.relics:
                        if z.id[-1:] == str(position):
                            result["relict"] = [await relicts.creat(z,character.id,position,name_charter= character.name)]
                else:
                    relic_tasks = [relicts.creat(key,character.id,key.id[-1:], name_charter= character.name) for key in character.relics]
                    result["relict"] = await asyncio.gather(*relic_tasks)

        if not position is None:
            result["card"] = result["relict"][0].card
        else:
            positione = [
                (40,40),(593,40),
                (40,401),(593,401),
                (1146,40),(1146,401),
            ]

            card_bg = openFile.ImageCache().relict_total_bg.convert("RGBA")

            for key in result["relict"]:
                card_bg.alpha_composite(key.card, positione[key.position-1])
            
            result["card"] = card_bg

        return modal.StarRailRelict(**result)


    async def add_author(self, card, link = "", name = "", profile = False):
        """
        :param card: PILL.IMAGE, Generated card using the create function
        :param link: str, Specify the author through the link
        :param name: str, Specify the author through the text
        :param profile: bool, Is the card a profile card.
        """
        types = 0
        if link != "":
            try:
                _, text = await author.get_site_info(link)
                icon = await author.get_site_icon(link)
            except:
                url = link
                text = link
                icon = openFile.ImageCache().icon
                types = 1
        else:
            if name != "":
                text = name
                icon = openFile.ImageCache().icon
                types = 1
            else:
                raise TypeError("Specify one of the parameters: lang or name")

        author_icon = await author.start(text,icon,types)

        if profile:
            position = (510,0)
        else:
            if self.template == 1:
                position = (278,447)
            elif self.template == 2:
                position = (36,56)
            else:
                position = (1356,609)

        card.alpha_composite(author_icon,position)
        
        return card
            

class StarRaillScore:
    def __init__(self, chart_id, relict) -> None:
        """
        :param chart_id: int, Character id
        :param relict: class, the object you get from the method: get_relict()
        """
        
        if relict == []:
            raise TypeError("Doesn't accept lists. Only 1 relic")
        
        if not relict.position:
            raise TypeError("Pass an object of the RelictData class, you can get it using the method: get_relict()")
        
        self.relict = relict
        self.chart_id = chart_id

    async def __aenter__(self):
        return self

    async def __aexit__(self, *args):
        pass

    async def calculate(self):
        return await relicts.creat(self.relict.relict,self.chart_id,self.relict.position)



# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.
from PIL import ImageFont,Image,ImageDraw,ImageChops,ImageFilter,ImageStat
from io import BytesIO
from . import openFile
import aiohttp,re, json
import colorsys
from cachetools import TTLCache
import numpy as np
import colorsys
from more_itertools import chunked

async def get_font(size):
    return ImageFont.truetype(openFile.font, size)


xId = "91470304"
ccokie = "first_visit_datetime_pc=2022-08-06+03:53:37; p_ab_id=1; p_ab_id_2=5; p_ab_d_id=1897822829; yuid_b=IFV4MVY; privacy_policy_agreement=5; c_type=23; privacy_policy_notification=0; a_type=0; b_type=1; __utmc=235335808; __utmz=235335808.1675712222.1.1.utmcsr=(direct)|utmccn=(direct)|utmcmd=(none); _gcl_au=1.1.1586580017.1675934633; _gid=GA1.2.67267752.1677021212; PHPSESSID=91470304_hbEoBFwL6Ss8hQHDiSkc26NAN2BgUaww; device_token=cbf72f380348bc4dcc9910df20a3b368; QSI_S_ZN_5hF4My7Ad6VNNAi=v:100:0; __utmv=235335808.|2=login ever=yes=1^3=plan=premium=1^5=gender=male=1^6=user_id=91470304=1^9=p_ab_id=1=1^10=p_ab_id_2=5=1^11=lang=en=1; _ga_MZ1NL4PHH0=GS1.1.1677021212.1.1.1677021635.0.0.0; __utma=235335808.1013236179.1675712222.1677021201.1677023923.4; login_ever=yes; __cf_bm=uIwLHChsA9lvfHUYdc_qU3KBp.pYrFxzrlv_4crFoE4-1677024971-0-AaFldWtGUM9OmDn1Kfcwc03QpGNuGGlE8Ev1PZtv6Q6PavyffvJ2dmVVIDVdeTM6cD8GNSLlL8ta93GxurhWiQqj+rxXEWgO3LDUqV0uXNORvDhI4+KP930Hf962s6ivFp1Zz6aG5fVGtpySkJBAEcVUAoxfpO6+KGijUP4sJAvftKvKK8NZaD6zcqDr47mOMJsHCvdck/DW4GqbSDeuIJo=; __utmt=1; tag_view_ranking=_EOd7bsGyl~ziiAzr_h04~azESOjmQSV~Ie2c51_4Sp~Lt-oEicbBr~WMwjH6DkQT~HY55MqmzzQ~yREQ8PVGHN~MnGbHeuS94~BSlt10mdnm~tgP8r-gOe_~fg8EOt4owo~b_rY80S-DW~1kqPgx5bT5~5oPIfUbtd6~KN7uxuR89w~QaiOjmwQnI~0Sds1vVNKR~pA1j4WTFmq~aPdvNeJ_XM~vzTU7cI86f~HHxwTpn5dx~pnCQRVigpy~eVxus64GZU~rOnsP2Q5UN~-98s6o2-Rp~EZQqoW9r8g~iAHff6Sx6z~jk9IzfjZ6n~PsltMJiybA~TqiZfKmSCg~IfWbVPYrW4~0TgyeJ7TQv~g2IyszmEaU~28gdfFXlY7~DCzSewSYcl~n15dndrA2h~CActc_bORM~U51WZv5L6G~-7RnTas_L3~zyKU3Q5L4C~QwUeUr8yRJ~j3leh4reoN~vgqit5QC27~t1Am7AQCDs~5cTBH7OrXg~-HnQonkV01~oCqKGRNl20~ba025Wj3s2~TAc-DD8LV2~p0NI-IYoo2~wqBB0CzEFh~U-RInt8VSZ~oiDfuNWtp4~fAWkkRackx~i54EuUSPdz~Js5EBY4gOW~ZQJ8wXoTHu~Cm1Eidma50~CMvJQbTsDH~ocDr8uHfOS~pzZvureUki~ZNRc-RnkNl~nWC-P2-9TI~q1r4Vd8vYK~hZzvvipTPD~DpYZ-BAzxm~096PrTDcN1~3WI2JuKHdp~faHcYIP1U0~1n-RsNEFpK~Bd2L9ZBE8q~txZ9z5ByU7~r01unnQL0a~EEUtbD_K_n~cb-9gnu4GK~npWJIbJroU~XbjPDXsKD-~lkoWqucyTw~P8OX_Lzc1b~RmnFFg7HS4~6rYZ-6JKHq~d80xTahBd1~OYl5wlor4w~2R7RYffVfj~1CWwi2xr7g~c7QmKEJ54V~rlExNugdTH~wO2lnVhO8m~vc2ipXnqbX~Is5E1jIZcw~c_aC4uL3np~vzxes78G4k; _ga=GA1.2.714813637.1675712223; _gat_UA-1830249-3=1; _ga_75BBYNYN9J=GS1.1.1677023923.4.1.1677025390.0.0.0; __utmb=235335808.52.9.1677024704913"

headers = {
    "accept-type": "application/json",
    "accept-encoding": "ru,en-US;q=0.9,en;q=0.8,uk;q=0.7,af;q=0.6",
    "language": "gzip, deflate, br",
    "cookie": ccokie,
    "dnt": "1",
    "sec-ch-ua-mobile": "?0",
    "sec-ch-ua-platform": "Windows",
    "sec-fetch-dest": "empty",
    "sec-fetch-mode": "cors",
    "sec-fetch-site": "same-origin",
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.102 Safari/537.36",
    "x-user-id": xId,
    "referer": "https://www.pixiv.net/",
}

cache = TTLCache(maxsize=1000, ttl=300)  

async def get_dowload_img(link,size = None, thumbnail_size = None):
    cache_key = json.dumps((link, size, thumbnail_size), sort_keys=True)  # Преобразовываем в строку
        
    if cache_key in cache:
        return cache[cache_key]
    

    try:
        if "pximg" in link:
            async with aiohttp.ClientSession(headers=headers) as session, session.get(link) as r:
                try:
                    image = await r.read()
                finally:
                    await session.close()
        else:
            async with aiohttp.ClientSession() as session:
                async with session.get(link) as response:
                    try:
                        image = await response.read()
                    finally:
                        await session.close()
    except:
        raise
    try:
        image = Image.open(BytesIO(image)).convert("RGBA")
    except:
        #print(link)
        image = Image.new("RGBA",(1,1),(0,0,0,0))
    if size:
        image = image.resize(size)
        cache[cache_key] = image
        return image
    elif thumbnail_size:
        image.thumbnail(thumbnail_size)
        cache[cache_key] = image
        return image
    else:
        cache[cache_key] = image
        return image


async def get_user_image(img):
    if type(img) != str:
        img = img
    elif type(img) == str:
        linkImg = re.search("(?P<url>https?://[^\s]+)", img)
        if linkImg:
            try:
                img = await get_dowload_img(linkImg.group())
            except:
                return None
        else:
            img = Image.open(img)
    else:
        return None
    return img.convert("RGBA")

async def get_resize_image(userImages,baseheight,basewidth):
    x,y = userImages.size
    if max(x, y) / min(x, y) < 1.1:
        hpercent = (baseheight / float (y)) 
        wsize = int ((float (x) * float (hpercent)))
        userImages = userImages.resize ((wsize, baseheight), Image.LANCZOS)
        return {"img": userImages, "type": 0}
    elif x > y:
        hpercent = (baseheight / float (y)) 
        wsize = int ((float (x) * float (hpercent)))
        userImages = userImages.resize ((wsize, baseheight), Image.LANCZOS)
        return {"img": userImages, "type": 1}
    else:
        wpercent = (basewidth / float(userImages.size[0]))
        hsize = int((float(userImages.size[1]) * float(wpercent)))
        userImages = userImages.resize((basewidth, hsize), Image.LANCZOS)
        if hsize < baseheight:
            hpercent = (baseheight / float (y)) 
            wsize = int ((float (x) * float (hpercent)))
            userImages = userImages.resize ((wsize, baseheight), Image.LANCZOS)
            return {"img": userImages, "type": 2}
        return {"img": userImages, "type": 2}


async def get_text_size_frame(text,font_size,frame_width):
    font = await get_font(font_size)

    while font.getlength(text) > frame_width:
        font_size -= 1
        font = await get_font(font_size)

    return font,font.getlength(text)


async def create_image_text(text, font_size, max_width=336, max_height=None, color=(255, 255, 255, 255)):
    original_font = await get_font(font_size)
    font = original_font
    lines = []
    line = []
    for word in text.split():
        if line:
            temp_line = line + [word]
            temp_text = ' '.join(temp_line)
            temp_width = font.getmask(temp_text).getbbox()[2]
            if temp_width <= max_width:
                line = temp_line
            else:
                lines.append(line)
                line = [word]
        else:
            line = [word]
    if line:
        lines.append(line)

    width = 0
    height = 0
    for line in lines:
        line_width = font.getmask(' '.join(line)).getbbox()[2]
        width = max(width, line_width)
        height += font.getmask(' '.join(line)).getbbox()[3]

    if max_height is not None and height > max_height:
        reduction_ratio = max_height / height
        new_font_size = int(font_size * reduction_ratio)
        font = await get_font(new_font_size)

    img = Image.new('RGBA', (min(width, max_width), height + (font_size - 4)), color=(255, 255, 255, 0))

    draw = ImageDraw.Draw(img)
    y_text = 0
    for line in lines:
        text_width, text_height = font.getmask(' '.join(line)).getbbox()[2:]
        draw.text((0, y_text), ' '.join(line), font=font, fill=color)
        y_text += text_height + 5

    return img

async def create_image_with_text(text, font_size, max_width=336, color=(255, 255, 255, 255), alg="Left"):
    font = await get_font(font_size)

    lines = []
    line = []
    for word in text.split():
        if line:
            temp_line = line + [word]
            temp_text = ' '.join(temp_line)
            temp_width = font.getmask(temp_text).getbbox()[2]
            if temp_width <= max_width:
                line = temp_line
            else:
                lines.append(line)
                line = [word]
        else:
            line = [word]
    if line:
        lines.append(line)

    width = 0
    height = 0
    for line in lines:
        line_width = font.getmask(' '.join(line)).getbbox()[2]
        width = max(width, line_width)
        height += font.getmask(' '.join(line)).getbbox()[3]

    img = Image.new('RGBA', (min(width, max_width), height + (font_size)), color=(255, 255, 255, 0))

    draw = ImageDraw.Draw(img)
    
    y_text = 0
    for line_num, line in enumerate(lines):
        text_width, text_height = font.getmask(' '.join(line)).getbbox()[2:]
        if alg == "center" and line_num > 0:
            x_text = (max_width - text_width) // 2
        else:
            x_text = 0
        draw.text((x_text, y_text), ' '.join(line), font=font, fill=color)
        y_text += text_height + 5

    return img

async def resize_image(image, scale):
    width, height = image.size
    new_width = int(width * scale)
    new_height = int(height * scale)
    resized_image = image.resize((new_width, new_height))
    return resized_image

async def get_centr_honkai(size, file_name):
    # Открываем фоновое изображение и изображение для наложения
    background_image = Image.new('RGBA', size, color=(0, 0, 0, 0))
    foreground_image = file_name.convert("RGBA")

    # Уменьшаем исходное изображение на 10%
    scale = 0.65
    foreground_image = await resize_image(foreground_image, scale)

    background_size = background_image.size
    foreground_size = foreground_image.size

    # Определяем позицию для наложения изображения
    x = background_size[0] // 2 - foreground_size[0] // 2
    y = background_size[1] // 2 - foreground_size[1] // 2

    # Налагаем изображение на фон
    background_image.alpha_composite(foreground_image, (x, y))

    return background_image

async def creat_bg_teample_two(image,bg,maska = None, teample = 2):
    if teample == 3:
        bg_element = bg.convert("RGBA")
        splash = await get_dowload_img(link=image)
        frme_splash = await get_centr_honkai((582,802),splash)
        frme_splash.alpha_composite(openFile.ImageCache().shadow_art,(0,0))
        bg_element.alpha_composite(frme_splash,(1342,0))
    elif teample == 4:
        splash = await get_dowload_img(link=image)
        frme_splash = await get_centr_honkai((685,802),splash)
        bg_frame = Image.new('RGBA', (685,802), color= (255,255,255,0))
        bg_frame.alpha_composite(frme_splash,(0,0))
        bg_element = bg.copy().convert("RGBA")
        maska_bg =  bg.copy()
        bg_element.alpha_composite(bg_frame,(0,0))
        bg_element.paste(maska_bg,(0,0),maska)
        bg_element.alpha_composite(openFile.ImageCache().shadow_enc,(0,0))
    elif teample == 5:
        splash = await get_dowload_img(link=image)
        frme_splash = await get_centr_honkai((370,663),splash)
        bg_element = Image.new('RGBA', (370,663), color= (255,255,255,0))
        bg_element.paste(frme_splash,(0,0),maska)
        bg.alpha_composite(bg_element,(0,0))        
        return bg
    else:
        splash = await get_dowload_img(link=image)
        frme_splash = await get_centr_honkai((719,719),splash)
        bg_frame = Image.new('RGBA', (1916,719), color= (255,255,255,0))
        bg_frame.alpha_composite(frme_splash,(0,0))
        bg_element = bg.copy()
        maska_bg =  bg.copy()
        bg_element.alpha_composite(bg_frame,(0,0))
        bg_element.paste(maska_bg,(0,0),maska)
        bg_element.alpha_composite(openFile.ImageCache().shadow,(2,0))

    return bg_element

async def light_level(pixel_color):
    h, l, s = colorsys.rgb_to_hls(*(x / 255 for x in pixel_color[:3]))    
    return l

async def recolor_image(image, target_color, light = False):
    if light:
        ll = await light_level(target_color)
        if ll < 45:
            target_color = await _get_light_pixel_color(target_color,up = True)
    if image.mode != 'RGBA':
        image = image.convert('RGBA')

    image = image.copy()

    pixels = image.load()
    for i in range(image.width):
        for j in range(image.height):
            r, g, b, a = pixels[i, j]
            if a != 0:  # Проверяем, является ли текущий пиксель непрозрачным
                pixels[i, j] = target_color + (a,)  # Заменяем цвет, включая прозрачность
    if light:
        return image, target_color
    return image


async def _get_light_pixel_color(pixel_color, up = False):
    h, l, s = colorsys.rgb_to_hls(*(x / 255 for x in pixel_color[:3]))
    if up:
        l = min(max(0.6, l), 0.9)
    else:
        l = min(max(0.3, l), 0.8)
    return tuple(round(x * 255) for x in colorsys.hls_to_rgb(h, l, s))
  
async def _get_dark_pixel_color(pixel_color):
    h, l, s = colorsys.rgb_to_hls(*(x / 255 for x in pixel_color[:3]))
    l = min(max(0.8, l), 0.2)
    a = tuple(round(x * 255) for x in colorsys.hls_to_rgb(h, l, s))
    
    return  a


class GradientGenerator:
    def __init__(self, source_img_path):
        self.source_img = source_img_path
        self.frame = ()
        self.source_width, self.source_height = self.source_img.size

    async def generate(self, width, height, left = False):
        gradient_img = Image.new("RGB", (width, height))
        top_height = height // 3
        bottom_height = height // 3
        center_height = height - top_height - bottom_height
        if left:
            left = 3
            right = 4
        else:
            left = self.source_width - 142
            right = self.source_width - 141
        top_1 = 1
        bottom_1 = top_height - 1
        top_2 = top_height + 1
        bottom_2 = top_height + center_height - 1
        top_3 = top_height + center_height + 1
        bottom_3 = height - 2

        top_color = await self._get_pixel_color(left, top_1, right, bottom_1)
        ll = await light_level(top_color)
        if ll < 45:
            top_color = await _get_light_pixel_color(top_color)
        elif ll > 200:
            top_color = await _get_dark_pixel_color(top_color)

        center_color = await self._get_pixel_color(left, top_2, right, bottom_2)
        
        ll = await light_level(center_color)
        if ll < 45:
            center_color = await _get_light_pixel_color(center_color)
        elif ll > 200:
            center_color = await _get_dark_pixel_color(center_color)

        bottom_color = await self._get_pixel_color(left, top_3, right, bottom_3)
        ll = await light_level(bottom_color)
        if ll < 45:
            bottom_color = await _get_light_pixel_color(bottom_color)
        elif ll > 200:
            bottom_color = await _get_dark_pixel_color(bottom_color)

        for y in range(top_height):
            for x in range(width):
                ratio = y / (top_height - 1)
                gradient_color = self._get_interpolated_color(top_color, center_color, ratio)
                gradient_img.putpixel((x, y), gradient_color)

        for y in range(center_height):
            for x in range(width):
                ratio = y / (center_height - 1)
                gradient_color = self._get_interpolated_color(center_color, bottom_color, ratio)
                gradient_img.putpixel((x, y + top_height), gradient_color)

        for y in range(bottom_height):
            for x in range(width):
                gradient_color = bottom_color
                gradient_img.putpixel((x, y + top_height + center_height), gradient_color)

        return gradient_img

    async def _get_pixel_color(self, left, top, right, bottom):
        cropped_img = self.source_img.crop((left, top, right, bottom))
        resized_img = cropped_img.convert("RGB").resize((1, 1))
        pixel_color = resized_img.getpixel((0, 0))
        
        return pixel_color
    
    def _get_interpolated_color(self, start_color, end_color, ratio):
        return tuple(int(start_color[i] + (end_color[i] - start_color[i]) * ratio) for i in range(3))

async def apply_opacity(image, opacity=0.2):
    result_image = image.copy()
    alpha = result_image.split()[3]
    alpha = alpha.point(lambda p: int(p * opacity))
    result_image.putalpha(alpha)

    return result_image

class ImageCreat():
    def __init__(self, size, source_img):
        self.source_img = source_img
        self.frame = Image.new("RGBA", size, (0, 0, 0, 0))
        self.resized_img = None

    async def get_centry_image(self, baseheight=0, basewidth=0, baseheight_wide=None, centry=776):
        if self.resized_img is None:
            self.resized_img = self.resize_image(baseheight, basewidth, baseheight_wide)

        x, y = self.resized_img.size
        position = (0, 0)

        if centry != 0:
            position = (int(centry - x / 2), 0)

        self.frame.alpha_composite(self.resized_img, position)

        return self.frame

    def resize_image(self, baseheight=0, basewidth=0, baseheight_wide=None):
        x, y = self.source_img.size

        if baseheight_wide is None:
            baseheight_wide = baseheight

        if max(x, y) / min(x, y) < 1.1:
            # Квадрат
            hpercent = baseheight / float(y)
            wsize = int(float(x) * float(hpercent))
            resized_img = self.source_img.resize((wsize, baseheight), Image.LANCZOS)
        elif x > y:
            # Широкоформат
            hpercent = baseheight_wide / float(y)
            wsize = int(float(x) * float(hpercent))
            resized_img = self.source_img.resize((wsize, baseheight_wide), Image.BICUBIC)
        else:
            # Вертикаль
            wpercent = basewidth / float(x)
            hsize = int(float(y) * float(wpercent))
            resized_img = self.source_img.resize((basewidth, hsize), Image.LANCZOS)

            if hsize < baseheight:
                hpercent = baseheight / float(y)
                wsize = int(float(x) * float(hpercent))
                resized_img = self.source_img.resize((wsize, baseheight), Image.BICUBIC)

        return resized_img




async def creat_user_image(img, teampl = "1", shadow = None, bg = None):
    position = {
        "1":
        {
            "frame": (769,719), #Область изображения
            "size_teample": (1916,719), #Размер всего изображения
            "baseheight": 770, #Квадрат
            "basewidth": 769, #Вертикально
            "baseheight_wide": None, #Широкий
            "centry": 385,
            "start_x": 0,
            "start_y": 0,
            "mask": openFile.ImageCache().MASKA_ART_CUSTUM.convert("L"), 
            "effect": openFile.ImageCache().effect_stars.convert("RGBA") 
        },
    }.get(teampl)
    frame = Image.new("RGBA", position["size_teample"], (0,0,0,0))
    userImagess = await ImageCreat(position["frame"],img).get_centry_image(baseheight = position["baseheight"], basewidth = position["basewidth"], baseheight_wide = position["baseheight_wide"], centry = position["centry"])
    grandient = await GradientGenerator(userImagess).generate(1,position["size_teample"][1])
    grandient = grandient.resize(position["size_teample"])
    grandient = ImageChops.soft_light(grandient.convert("RGBA"), position["effect"])

    frame.alpha_composite(userImagess,(position["start_x"],position["start_y"]))


    bg.paste(frame,(0,0), openFile.ImageCache().MASKA_ARTS.convert("L"))

    bg.paste(grandient,(0,0),position["mask"])

    bg.alpha_composite(shadow,(0,0))
    
    return bg

async def creat_user_image_tree(img,characterBackgroundimg,backgroundBlur):
    bg = Image.new("RGBA", (1924,802), (36,36,36,255))
    userImagess = await ImageCreat((582,802),img).get_centry_image(baseheight = 802, basewidth = 582, baseheight_wide = None, centry = 291)
    if characterBackgroundimg is None:
        grandient = await GradientGenerator(userImagess).generate(1,802, left= True)
        grandient = grandient.resize((1350,802))
        bg.alpha_composite(grandient.convert("RGBA"),(0,0))
        grandient = ImageChops.soft_light(bg, openFile.ImageCache().overlay.convert("RGBA"))
        grandient.alpha_composite(openFile.ImageCache().art_frame,(0,0))
        userImagess.alpha_composite(openFile.ImageCache().shadow_art,(0,0))
        grandient.alpha_composite(userImagess,(1342,0))
    else:
        #ТУТ ДОБАВЛЯЕМ ФОН ПОЛЬЗОВАТЕЛЯ
        if backgroundBlur:
            shadow = Image.new("RGBA", (1350,802), (0,0,0,20))
            bgmagess = await ImageCreat((1350,802),characterBackgroundimg).get_centry_image(baseheight = 802, basewidth = 1350, baseheight_wide = 900, centry = 675)
            grandient = bgmagess.filter(ImageFilter.GaussianBlur(5))
            grandient.alpha_composite(shadow,(0,0))
            bg.alpha_composite(grandient.convert("RGBA"),(0,0))
            grandient = ImageChops.soft_light(bg, openFile.ImageCache().overlay.convert("RGBA"))
            grandient.alpha_composite(openFile.ImageCache().art_frame,(0,0))
            userImagess.alpha_composite(openFile.ImageCache().shadow_art,(0,0))
            grandient.alpha_composite(userImagess,(1342,0))
            #ТУТ РАЗМЫВАЕМ и ДОБАВЛЕМ ТЕНЬ
        else:
            shadow = Image.new("RGBA", (1350,802), (0,0,0,20))
            grandient = await ImageCreat((1350,802),characterBackgroundimg).get_centry_image(baseheight = 802, basewidth = 1350, baseheight_wide = 900, centry = 675)
            grandient.alpha_composite(shadow,(0,0))
            bg.alpha_composite(grandient.convert("RGBA"),(0,0))
            grandient = ImageChops.soft_light(bg, openFile.ImageCache().overlay.convert("RGBA"))
            grandient.alpha_composite(openFile.ImageCache().art_frame,(0,0))
            userImagess.alpha_composite(openFile.ImageCache().shadow_art,(0,0))
            grandient.alpha_composite(userImagess,(1342,0))

    return grandient

async def creat_user_image_five(img):
    bg = Image.new("RGBA", (370,663), (0,0,0,0))
    userImagess = await ImageCreat((370,663),img).get_centry_image(baseheight = 663, basewidth = 370, baseheight_wide = None, centry = 185)
    bg.paste(userImagess,(0,0),openFile.ImageCache().mask_bg_five.convert("L"))
    bg.alpha_composite(openFile.ImageCache().shadow_bg_five,(0,0))
    
    return bg
    

async def creat_user_image_four(img):
    bg = Image.new("RGBA", (1924,802), (0,0,0,0))
    userImagess = await get_centr_honkai_art((685,802),img) #await ImageCreat((685,803),img).get_centry_image(baseheight = 1082, basewidth = 582, baseheight_wide = None, centry = 342)
    grandient = await GradientGenerator(userImagess).generate(1,802)
    grandient = grandient.resize((1924,802)).convert("RGBA")
    
    grandient = ImageChops.soft_light(grandient, openFile.ImageCache().effect_soft.convert("RGBA"))
    bg.alpha_composite(grandient,(0,0))
    bg.alpha_composite(userImagess,(0,0))
    bg.paste(grandient.resize((1924,802)),(0,0),openFile.ImageCache().maska_background.convert("L"))
    bg.alpha_composite(openFile.ImageCache().shadow_enc,(0,0))
    return bg

async def get_centr_honkai_art(size, file_name):
    background_image = Image.new('RGBA', size, color=(0, 0, 0, 0))
    foreground_image = file_name.convert("RGBA")

    scale = max(size[0] / foreground_image.size[0], size[1] / foreground_image.size[1])
    foreground_image = foreground_image.resize((int(foreground_image.size[0] * scale), int(foreground_image.size[1] * scale)))

    background_size = background_image.size
    foreground_size = foreground_image.size

    x = background_size[0] // 2 - foreground_size[0] // 2

    if foreground_size[1] > background_size[1]:
        y_offset = max(int(0.3 * (foreground_size[1] - background_size[1])), int(0.5 * (-foreground_size[1])))
        y = -y_offset
    else:
        y = background_size[1] // 2 - foreground_size[1] // 2

    background_image.alpha_composite(foreground_image, (x, y))

    return background_image



async def apply_blur_and_overlay(img):
    background = Image.new("RGBA", (681, 459), (0, 0, 0, 0))

    overlay = await get_centr_honkai_art((694, 802),img)

    overlay_blurred = overlay.filter(ImageFilter.GaussianBlur(radius=10))

    background.alpha_composite(overlay_blurred,(0,-343))

    return background



async def get_average_color(image):
    if image.mode != 'RGBA':
        image = image.convert('RGBA')
    
    channels = image.split()
    
    return (
        round(np.average(channels[0], weights=channels[-1])),
        round(np.average(channels[1], weights=channels[-1])),
        round(np.average(channels[2], weights=channels[-1])),
    )


async def get_dominant_colors(
    image,
    number,
    *,
    dither=Image.Quantize.FASTOCTREE,
    common=True,
):
    if image.mode != 'RGB':
        if image.mode != 'RGBA':
            image = image.convert('RGBA')
        
        if not common:
            width = image.width
            height = image.height
            
            image = Image.fromarray(np.array([np.repeat(
                np.reshape(image.convert('RGB'), (width * height, 3)),
                np.reshape(image.split()[-1], width * height),
                0,
            )]), 'RGB')
    
    if image.mode == 'RGBA':
        if dither == Image.Quantize.FASTOCTREE:
            simple_image = image.copy()
            simple_image.putalpha(255)
        else:
            simple_image = image.convert('RGB')
    else:
        simple_image = image
    
    reduced = simple_image.quantize(dither=dither, colors=number)
    
    palette = [*chunked(reduced.getpalette(), 3)]
    
    if common and image.mode == 'RGBA':
        alpha = np.array(image.split()[-1])
        
        colors = sorted((
            (
                np.sum(alpha * reduced.point([0] * i + [1] + [0] * (255 - i))),
                tuple(palette[i]),
            )
            for _, i in reduced.getcolors()
        ), reverse=True)
    else:
        colors = [
            (n, tuple(palette[i]))
            for n, i in sorted(reduced.getcolors(), reverse=True)
        ]
    
    return tuple(colors)


async def get_distance_alpha(image, converter=(lambda x: x)):
    width = image.width
    height = image.height
    
    radius = np.hypot(1, 1)
    
    return Image.fromarray(np.fromfunction(
        lambda y, x: np.uint8(255 * converter(np.hypot(
            2 * x / (width - 1) - 1,
            2 * y / (height - 1) - 1,
        ) / radius)),
        (height, width),
    ), 'L')


async def get_background_alpha(image):
    return await get_distance_alpha(
        image,
        lambda x: x * np.sin(x * np.pi / 2),
    )


async def get_foreground_alpha(image):
    return await get_distance_alpha(
        image,
        lambda x: 1 - x * np.sin(x * np.pi / 2),
    )

async def get_background_colors(image,number,*,common=False,radius=1,quality=None):
    if quality is not None:
        image = image.copy()
        image.thumbnail((quality, quality), 0)
    
    if radius > 1:
        image = image.filter(ImageFilter.BoxBlur(radius))
    
    filtered_image = image.convert('RGB')
    
    if image.mode != 'RGBA':
        filtered_image.putalpha(await get_background_alpha(image))
    else:
        filtered_image.putalpha(Image.fromarray(np.uint8(
            np.uint16(await get_background_alpha(image))
            * image.split()[-1]
            / 255
        ), 'L'))
    
    color_palette = await get_dominant_colors(filtered_image, number, common=common)
    color_palette = color_palette[0][1]
    ll = await light_level(color_palette)
    if ll < 0.15:
        color_palette = await _get_light_pixel_color(color_palette)
    elif ll > 0.80:
        color_palette = await _get_dark_pixel_color(color_palette)
        
        
    return color_palette
     


async def get_foreground_colors(image,number,*,common=False,radius=1,quality=None):
    if quality is not None:
        image = image.copy()
        image.thumbnail((quality, quality), 0)
    
    if radius > 1:
        image = image.filter(ImageFilter.BoxBlur(radius))
    
    filtered_image = image.convert('RGB')
    
    if image.mode != 'RGBA':
        filtered_image.putalpha(await get_foreground_alpha(image))
    else:
        filtered_image.putalpha(Image.fromarray(np.uint8(
            np.uint16(await  get_foreground_alpha(image))
            * image.split()[-1]
            / 255
        ), 'L'))
    
    return await  get_dominant_colors(filtered_image, number, common=common)


# Copyright 2023 DEViantUa <t.me/deviant_ua>
# All rights reserved.

import aiohttp

url = "https://raw.githubusercontent.com/Mar-7th/StarRailScore/master/score.json"

async def fetch_data():
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            data = await response.json(content_type='text/plain')
            return data
        

data = None

async def get_rank(value):
    if value >= 43.7:
        return "SSS"
    elif value >= 37.8 and value < 43.7:
        return "SS"
    elif value >= 33.5 and value  < 37.8:
        return "S"
    elif value >= 29.1 and value < 33.5:
        return "A"
    elif value >= 23.3 and value  < 29.1:
        return "B"
    elif value >= 17.5 and value  < 23.3:
        return "C"
    elif value >= 11.6 and value  < 17.5:
        return "D"
    else:
        return "N/A"

async def get_total_rank(value):
    value /= 6
    if value >= 43.7:
        return "SSS"
    elif value >= 37.8 and value < 43.7:
        return "SS"
    elif value >= 33.5 and value  < 37.8:
        return "S"
    elif value >= 29.1 and value < 33.5:
        return "A"
    elif value >= 23.3 and value  < 29.1:
        return "B"
    elif value >= 17.5 and value  < 23.3:
        return "C"
    elif value >= 11.6 and value  < 17.5:
        return "D"
    else:
        return "N/A"



async def get_rating(relict,chart_id, position):
    global data
    if data is None:
        data = await fetch_data()

    Eff_Stat = 0
    value_main = 0
    if str(chart_id):
        if str(chart_id) in data:
            if relict.main_affix.type in data[str(chart_id)]["main"][position]:
                value_main = data[str(chart_id)]["main"][position][relict.main_affix.type]
        
            i = 0
            score = 0
            for key in relict.sub_affix:
                value = 0
                if key.type in data[str(chart_id)]["weight"]:
                    if data[str(chart_id)]["weight"][key.type] > 0:
                        Eff_Stat += 1
                    value = data[str(chart_id)]["weight"][key.type]

                i += 1
                score += value

            score += Eff_Stat * value_main
            score = 55/10.0*score

            return score, await get_rank(score), Eff_Stat
    return 0, await get_rank(0), 0


from pydantic import BaseModel

class Affix(BaseModel):
    type: str
    field: str
    name: str
    icon: str
    value: float
    display: str
    percent: bool = False
    count: int = 0
    step: int = 0

class Relict(BaseModel):
    id: str
    name: str
    set_id: str
    set_name: str
    rarity: int
    level: int
    icon: str
    main_affix: Affix
    sub_affix: list[Affix]


async def fetch_data_seeleland(uid):
    url = f"https://seeleland.azurewebsites.net/api/get_player_data?uid={uid}"
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            data = await response.json()
            return data


async def get_seeleland(uid, charter_id):
    data = await fetch_data_seeleland(uid)
    if data is None:
        return None
    for key in data:
        if key["k"] == str(charter_id):
            data = key.get("lb")
    
    if data != {} and data is not None:
        for key in data:
            if isinstance(key, dict):
                return None
            return data[key]
    else:
        return None

    



from setuptools import setup, find_packages

setup(
    name='starrailcard',
    version='1.2.5',
    description='Module for generating Honkai Star Rail character cards',
    author='None',
    packages=find_packages(),
    install_requires=[
        'Pillow>=10.0.1',
        'honkairail>=1.1.4',
        'python>=3.9',
    ],
    extras_require={
        'dev': [
            'cachetools>=5.3.1',
        ]
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    keywords=[
        'honkai', 'cards', 'generation', 'honkaistarraill', 'raill', 'starraill',
        'builds', 'honkairail', 'honkai'
    ],
    url='https://github.com/DEViantUA/StarRailCard',
    project_urls={
        'Source': 'https://github.com/DEViantUA/StarRailCard',
        'Documentation': 'https://github.com/DEViantUA/StarRailCard/blob/main/README.md',
    },
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    package_data={
        'starrailcard': ['src/*'],
    },
)
